@@ -24,12 +24,13 @@
 #include <afs/budb_prototypes.h>
 #include <afs/butc.h>
 #include <afs/bubasics.h>	/* PA */
+#include <afs/afsint.h>
 #include <afs/volser.h>
 #include <afs/voldefs.h>	/* PA */
 #include <afs/vldbint.h>	/* PA */
 #include <afs/ktime.h>		/* PA */
 #include <ubik.h>
-#include <lock.h>
+#include <afs/afs_lock.h>
 #include <afs/tcdata.h>
 #include <afs/butx.h>
 #include <afs/vsutils_prototypes.h>
@@ -206,6 +207,7 @@ EvalVolumeSet2(struct bc_config *aconfig,
     *avols = (struct bc_volumeDump *)0;
     bulkentries.nbulkentries_len = 0;
     bulkentries.nbulkentries_val = 0;
+    memset(&attributes, 0, sizeof(attributes));
 
     /* For each of the volume set entries - collect the volumes that match it */
     for (tve = avs->ventries; tve; tve = tve->next) {
@@ -256,15 +258,15 @@ EvalVolumeSet2(struct bc_config *aconfig,
 		ei = entries[e].matchindex & 0xffff;
 		et = (entries[e].matchindex >> 16) & 0xffff;
 		switch (et) {
-		case ITSRWVOL:{
+		case VLSF_RWVOL:{
 			et = RWVOL;
 			break;
 		    }
-		case ITSBACKVOL:{
+		case VLSF_BACKVOL:{
 			et = BACKVOL;
 			break;
 		    }
-		case ITSROVOL:{
+		case VLSF_ROVOL:{
 			et = ROVOL;
 			break;
 		    }
@@ -501,8 +503,8 @@ EvalVolumeSet1(struct bc_config *aconfig,
 		/* If the RW name matches the volume set entry, take
 		 * it and exit. First choice is to use the RW volume.
 		 */
-		if (entry.serverFlags[srvpartpair] & ITSRWVOL) {
-		    if (entry.flags & RW_EXISTS) {
+		if (entry.serverFlags[srvpartpair] & VLSF_RWVOL) {
+		    if (entry.flags & VLF_RWEXISTS) {
 			sprintf(patt, "%s", entry.name);
 #ifdef HAVE_POSIX_REGEX
 			code = regexec(&re, patt, 0, NULL, 0);
@@ -521,7 +523,7 @@ EvalVolumeSet1(struct bc_config *aconfig,
 		    /* If the BK name matches the volume set entry, take
 		     * it and exit. Second choice is to use the BK volume.
 		     */
-		    if (entry.flags & BACK_EXISTS) {
+		    if (entry.flags & VLF_BACKEXISTS) {
 			sprintf(patt, "%s.backup", entry.name);
 #ifdef HAVE_POSIX_REGEX
 			code = regexec(&re, patt, 0, NULL, 0);
@@ -542,8 +544,8 @@ EvalVolumeSet1(struct bc_config *aconfig,
 		 * it, but continue searching. Further entries may be
 		 * RW or backup entries that will match.
 		 */
-		else if (!found && (entry.serverFlags[srvpartpair] & ITSROVOL)
-			 && (entry.flags & RO_EXISTS)) {
+		else if (!found && (entry.serverFlags[srvpartpair] & VLSF_ROVOL)
+			 && (entry.flags & VLF_ROEXISTS)) {
 		    sprintf(patt, "%s.readonly", entry.name);
 #ifdef HAVE_POSIX_REGEX
 		    code = regexec(&re, patt, 0, NULL, 0);
@@ -1161,21 +1163,17 @@ bc_VolRestoreCmd(struct cmd_syndesc *as, void *arock)
     }
 
     /* specified other destination host */
-    if (as->parms[0].items) {
-	tp = as->parms[0].items->data;
-	if (bc_ParseHost(tp, &destServ)) {
-	    afs_com_err(whoami, 0, "Failed to locate destination host '%s'", tp);
-	    return -1;
-	}
+    tp = as->parms[0].items->data;
+    if (bc_ParseHost(tp, &destServ)) {
+	afs_com_err(whoami, 0, "Failed to locate destination host '%s'", tp);
+	return -1;
     }
 
     /* specified other destination partition */
-    if (as->parms[1].items) {
-	tp = as->parms[1].items->data;
-	if (bc_GetPartitionID(tp, &destPartition)) {
-	    afs_com_err(whoami, 0, "Can't parse destination partition '%s'", tp);
-	    return -1;
-	}
+    tp = as->parms[1].items->data;
+    if (bc_GetPartitionID(tp, &destPartition)) {
+	afs_com_err(whoami, 0, "Can't parse destination partition '%s'", tp);
+	return -1;
     }
 
     for (ti = as->parms[2].items; ti; ti = ti->next) {
@@ -2448,7 +2446,7 @@ bc_deleteDumpCmd(struct cmd_syndesc *as, void *arock)
 	havetime = 1;
     }
 
-    port = (as->parms[3].items ? getPortOffset(as->parms[3].items->data) : 0);	/* -port */
+    port = (as->parms[3].items ? getPortOffset(as->parms[3].items->data) : 0);	/* -portoffset */
     if (as->parms[5].items)	/* -dbonly */
 	port = -1;
 
@@ -2727,6 +2725,8 @@ DBLookupByVolume(char *volumeName)
     char vname[BU_MAXNAMELEN];
     char ds[50];
 
+    memset(volumeEntry, 0, sizeof(volumeEntry));
+
     for (pass = 0; pass < 2; pass++) {
 	/*p */
 	/* On second pass, search for backup volume */
@@ -2939,13 +2939,18 @@ dumpInfo(afs_int32 dumpid, afs_int32 detailFlag)
 	    last = next;
 
 	    /* now get all the volumes in this dump. */
-	    code = ubik_Call_SingleServer(BUDB_GetVolumes, udbHandle.uh_client, UF_SINGLESERVER, BUDB_MAJORVERSION, BUDB_OP_DUMPID | BUDB_OP_TAPENAME, tapeLinkPtr->tapeEntry.name,	/* tape name */
-					  dumpid,	/* dumpid (not initial dumpid) */
-					  0,	/* end */
-					  last,	/* last */
-					  &next,	/* nextindex */
-					  &dbTime,	/* update time */
-					  &vl);
+	    code =
+		ubik_Call_SingleServer_BUDB_GetVolumes(udbHandle.uh_client,
+						       UF_SINGLESERVER,
+						       BUDB_MAJORVERSION,
+						       BUDB_OP_DUMPID | BUDB_OP_TAPENAME,
+						       tapeLinkPtr->tapeEntry.name,	/* tape name */
+						       dumpid,	/* dumpid (not initial dumpid) */
+						       0,	/* end */
+						       last,	/* last */
+						       &next,	/* nextindex */
+						       &dbTime,	/* update time */
+						       &vl);
 
 	    if (code) {
 		if (code == BUDB_ENDOFLIST) {	/* 0 volumes on tape */
