@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -18,7 +18,7 @@
  *
  *------------------------------------------------------------------------*/
 
-#include <lock.h>		/*AFS locks */
+#include <afs/afs_lock.h>		/*AFS locks */
 
 #define GATOR_TEXTCB_MAXINVERSIONS 10	/*Max highlight inversions */
 
